@@ -1,5 +1,9 @@
+# SPDX-FileCopyrightText: © 2021 Antonio López Rivera <antonlopezr99@gmail.com>
+# SPDX-License-Identifier: GPL-3.0-only
+
 """
-Plotting methods: 2D
+MPL Plotter: 2D plotting
+------------------------
 """
 
 from mpl_plotter.two_d.methods import line, scatter, heatmap, quiver, streamline, fill_area, floating_text, \
