@@ -1,4 +1,6 @@
+import re
 import inspect
+import warnings
 import numpy as np
 import pandas as pd
 import datetime as dt
@@ -10,11 +12,16 @@ from matplotlib import font_manager as font_manager
 from matplotlib.ticker import FormatStrFormatter
 
 from alexandria.shell import print_color
-from alexandria.data_structs.array import span
+from alexandria.data_structs.array import span, ensure_ndarray
 
 from mpl_plotter.two_d.mock import MockData
 
 
+# NumPy ufunc size changed warning override -
+# https://stackoverflow.com/questions/40845304/runtimewarning-numpy-dtype-size-changed-may-indicate-binary-incompatibility
+warnings.filterwarnings("ignore", message="numpy.ufunc size changed")
+
+
 class canvas:
 
     def method_backend(self):
@@ -91,9 +98,10 @@ class attributes:
                 None)) else self.workspace_color2
 
     def method_cb(self):
-        if self.color_bar is True:
+        pass
+        if self.color_bar:
             if isinstance(self.norm, type(None)):
-                return print_color("No norm selected for colorbar. Set norm=<parameter of choice>", "grey")
+                return print_color("No norm selected for colorbar. Set norm=<parameter of choice>", "black")
 
             # Obtain and apply limits
             if isinstance(self.cb_vmin, type(None)):
@@ -110,12 +118,9 @@ class attributes:
                 else self.cb_tick_ndecimals
             cbar = self.fig.colorbar(self.graph,
                                      ax=self.ax,
-                                     # Add option to have different colormap and colorbar ranges
-                                     norm=mpl.colors.Normalize(vmin=self.cb_vmin, vmax=self.cb_vmax),
-                                     # Add option to have different colormap and colorbar ranges
                                      orientation=self.cb_orientation, shrink=self.shrink,
                                      ticks=locator,
-                                     boundaries=locator if self.cb_hard_bounds is True else None,
+                                     boundaries=locator if self.cb_hard_bounds else None,
                                      spacing='proportional',
                                      extend=self.extend,
                                      format='%.' + str(cb_decimals) + 'f',
@@ -135,7 +140,7 @@ class attributes:
                 if not isinstance(self.cb_title,
                                   type(None)) and self.cb_y_title is False and self.cb_top_title is False:
                     print('Input colorbar title location with booleans: cb_y_title=True or cb_top_title=True')
-                if self.cb_y_title is True:
+                if self.cb_y_title:
                     cbar.ax.set_ylabel(self.cb_title, rotation=self.cb_title_rotation,
                                        labelpad=self.cb_ytitle_labelpad)
                     text = cbar.ax.yaxis.label
@@ -143,7 +148,7 @@ class attributes:
                                                            size=self.cb_title_size + self.font_size_increase,
                                                            weight=self.cb_title_weight)
                     text.set_font_properties(font)
-                if self.cb_top_title is True:
+                if self.cb_top_title:
                     cbar.ax.set_title(self.cb_title, rotation=self.cb_title_rotation,
                                       fontdict={'verticalalignment': 'baseline',
                                                 'horizontalalignment': 'left'},
@@ -167,7 +172,7 @@ class attributes:
             cbar.outline.set_linewidth(self.cb_outline_width)
 
     def method_legend(self):
-        if self.legend is True:
+        if self.legend:
             lines_labels = [ax.get_legend_handles_labels() for ax in self.fig.axes]
             lines, labels = [sum(lol, []) for lol in zip(*lines_labels)]
             legend_font = font_manager.FontProperties(family=self.font,
@@ -193,7 +198,7 @@ class attributes:
             if not isinstance(self.y_bounds[1], type(None)):
                 self.y_upper_bound = self.y_bounds[1]
 
-        if self.resize_axes is True:
+        if self.resize_axes and self.x.size != 0 and self.y.size != 0:
 
             def bounds(d, u, l, up, lp, v):
                 # Upper and lower bounds
@@ -228,7 +233,7 @@ class attributes:
                                                                                      self.y_bounds)
 
             # Room to breathe
-            if self.demo_pad_plot is True:
+            if self.demo_pad_plot:
                 pad_x = 0.05 * (abs(self.x + self.x.min()).max() - abs(self.x + self.x.min()).min())
                 self.x_upper_resize_pad = pad_x
                 self.x_lower_resize_pad = pad_x
@@ -310,7 +315,9 @@ class attributes:
 
             # y axis label rotation
             if isinstance(self.y_label_rotation, type(None)):
-                self.y_label_rotation = 90 if len(self.y_label) > 3 else 0
+                latex_chars  = re.findall(r'\$\\(.*?)\$', self.y_label)
+                label_length = len(self.y_label) - 2*len(latex_chars) - len(''.join(latex_chars).replace('//', '/'))
+                self.y_label_rotation = 90 if label_length > 3 else 0
 
             # Draw label
             self.ax.set_ylabel(self.y_label, fontname=self.font, weight=self.y_label_weight,
@@ -348,11 +355,12 @@ class attributes:
         if not isinstance(self.y, type(None)):  # Avoid issues with arrays with span 0 (vertical or horizontal lines)
             if span(self.y) == 0:
                 self.fine_tick_locations = False
-        if self.fine_tick_locations is True:
-            if not isinstance(self.x, type(None)) and isinstance(self.x_custom_tick_locations, type(None)):
-                self.x_custom_tick_locations = [self.x.min(), self.x.max()]
-            if not isinstance(self.y, type(None)) and isinstance(self.y_custom_tick_locations, type(None)):
-                self.y_custom_tick_locations = [self.y.min(), self.y.max()]
+        if not isinstance(self.x, type(None)) and not isinstance(self.y, type(None)):
+            if self.fine_tick_locations and self.x.size != 0 and self.y.size != 0:
+                if isinstance(self.x_custom_tick_locations, type(None)):
+                    self.x_custom_tick_locations = [self.x.min(), self.x.max()]
+                if isinstance(self.y_custom_tick_locations, type(None)):
+                    self.y_custom_tick_locations = [self.y.min(), self.y.max()]
         """
         Checks
         """
@@ -434,7 +442,7 @@ class attributes:
                                                         self.y_tick_number)
             self.ax.set_yticklabels(self.y_custom_tick_labels[::-1])
         #       Date tick labels
-        if self.x_date_tick_labels is True:
+        if self.x_date_tick_labels:
             fmtd = pd.date_range(start=self.x[0], end=self.x[-1], periods=self.x_tick_number)
             fmtd = [dt.datetime.strftime(d, self.date_format) for d in fmtd]
             self.ax.set_xticklabels(fmtd)
@@ -508,7 +516,7 @@ class plot(canvas, attributes):
             self.plt.savefig(self.filename, dpi=self.dpi)
 
     def method_show(self):
-        if self.show is True:
+        if self.show:
             self.fig.tight_layout()
             self.plt.show()
         else:
@@ -528,7 +536,7 @@ class std_input():
 class df_input:
 
     def method_resize_axes_dataframe(self):
-        if self.resize_axes is True:
+        if self.resize_axes:
             xmin = 0
             ymin = 0
             xmax = self.z.shape[0]
@@ -567,7 +575,7 @@ class line(plot, std_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -653,6 +661,10 @@ class line(plot, std_input):
         for item in inspect.signature(line).parameters:
             setattr(self, item, eval(item))
 
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
+
         self.init()
 
     def plot(self):
@@ -662,7 +674,7 @@ class line(plot, std_input):
                                       color=self.color,
                                       zorder=self.zorder,
                                       alpha=self.alpha,
-                                      )
+                                      )[0]
         else:
             # Create a set of line segments so that we can color them individually
             # This creates the points as a N x 1 x 2 array so that we can stack points
@@ -684,7 +696,7 @@ class line(plot, std_input):
     def mock(self):
         if isinstance(self.x, type(None)) and isinstance(self.y, type(None)):
             self.x, self.y = MockData().spirograph()
-            if self.norm is True:
+            if self.norm:
                 self.norm = self.y
 
 
@@ -701,7 +713,7 @@ class scatter(plot, std_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -788,6 +800,10 @@ class scatter(plot, std_input):
         for item in inspect.signature(scatter).parameters:
             setattr(self, item, eval(item))
 
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
+
         self.init()
 
     def plot(self):
@@ -822,7 +838,7 @@ class heatmap(plot, df_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -904,10 +920,16 @@ class heatmap(plot, df_input):
                         Plotting window freezes even if trying different backends with no backend error: python configuration problem
                             backend=None
         """
-        # Turn all instance arguments to instance attributes
+        # T
+        # urn all instance arguments to instance attributes
         for item in inspect.signature(heatmap).parameters:
             setattr(self, item, eval(item))
 
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
+        self.z = ensure_ndarray(self.z) if not isinstance(self.z, pd.DataFrame) else self.z if not isinstance(self.z, type(None)) else None
+
         self.init()
 
     def plot(self):
@@ -948,7 +970,7 @@ class quiver(plot, std_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -1036,10 +1058,15 @@ class quiver(plot, std_input):
                         Plotting window freezes even if trying different backends with no backend error: python configuration problem
                             backend=None
         """
-        # Turn all instance arguments to instance attributes
+        # T
+        # urn all instance arguments to instance attributes
         for item in inspect.signature(quiver).parameters:
             setattr(self, item, eval(item))
 
+
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
         self.init()
 
     def plot(self):
@@ -1098,7 +1125,7 @@ class streamline(plot, std_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -1188,6 +1215,12 @@ class streamline(plot, std_input):
         for item in inspect.signature(streamline).parameters:
             setattr(self, item, eval(item))
 
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
+        self.u = ensure_ndarray(self.u) if not isinstance(self.u, type(None)) else None
+        self.v = ensure_ndarray(self.v) if not isinstance(self.v, type(None)) else None
+
         self.init()
 
     def plot(self):
@@ -1232,7 +1265,7 @@ class fill_area(plot, std_input):
                  font='serif', math_font="dejavuserif", font_color="black", font_size_increase=0,
                  # Figure, axes
                  fig=None, ax=None, figsize=None, shape_and_position=111, prune=None, resize_axes=True,
-                 scale=None, aspect=1,
+                 scale=None, aspect=None,
                  # Setup
                  workspace_color=None, workspace_color2=None,
                  background_color_figure='white', background_color_plot='white', background_alpha=1,
@@ -1325,6 +1358,11 @@ class fill_area(plot, std_input):
         for item in inspect.signature(fill_area).parameters:
             setattr(self, item, eval(item))
 
+        # Ensure x and y are NumPy arrays
+        self.x = ensure_ndarray(self.x) if not isinstance(self.x, type(None)) else None
+        self.y = ensure_ndarray(self.y) if not isinstance(self.y, type(None)) else None
+        self.z = ensure_ndarray(self.z) if not isinstance(self.z, type(None)) else None
+
         self.init()
 
     def plot(self):
@@ -1333,13 +1371,13 @@ class fill_area(plot, std_input):
         Fill the region below the intersection of S and Z
         """
         if not isinstance(self.z, type(None)):
-            if self.between is True:
+            if self.between:
                 self.ax.fill_between(self.x, self.y, self.z, facecolor=self.color,
                                      alpha=self.alpha, label=self.plot_label)
-            if self.below is True:
+            if self.below:
                 self.ax.fill_between(self.x, self.i_below(), np.zeros(self.y.shape), facecolor=self.color,
                                      alpha=self.alpha, label=self.plot_label)
-            if self.above is True:
+            if self.above:
                 self.ax.fill_between(self.x, self.i_above(), np.zeros(self.y.shape), facecolor=self.color,
                                      alpha=self.alpha, label=self.plot_label)
             if self.between is False and self.below is False and self.above is False:
