@@ -1,3 +1,6 @@
+# SPDX-FileCopyrightText: © 2021 Antonio López Rivera <antonlopezr99@gmail.com>
+# SPDX-License-Identifier: GPL-3.0-only
+
 import unittest
 import numpy as np
 import matplotlib.pyplot as plt
@@ -41,7 +44,8 @@ class Test(unittest.TestCase):
                            y_tick_number=1,
                            z_tick_number=1,
                            grid=True,
-                           pane_fill="#f7f7f7"
+                           pane_fill="#f7f7f7",
+                           aspect_equal=True
                            )
         ax, fig = c.ax, c.fig
 
