@@ -1,3 +1,6 @@
+# SPDX-FileCopyrightText: © 2021 Antonio López Rivera <antonlopezr99@gmail.com>
+# SPDX-License-Identifier: GPL-3.0-only
+
 import unittest
 import numpy as np
 
@@ -22,11 +25,6 @@ class TestAll(unittest.TestCase):
 
         fill_area(show=show, backend=backend)
 
-        # Input
-        x = np.linspace(0, 2*np.pi, 100)
-        y = np.sin(x)
-        line(x=x, y=y, show=show, backend=backend, aspect=1)
-
     def test_three_d(self):
         from mpl_plotter.three_d import line, scatter, surface
 
