@@ -1,9 +1,11 @@
+# SPDX-FileCopyrightText: © 2021 Antonio López Rivera <antonlopezr99@gmail.com>
+# SPDX-License-Identifier: GPL-3.0-only
+
 import unittest
 
-import numpy as np
 import matplotlib.pyplot as plt
 
-from mpl_plotter.canvas import figure
+from mpl_plotter import figure
 from mpl_plotter.canvas import custom_canvas2
 from mpl_plotter.two_d import line, quiver, heatmap, streamline, fill_area
 from mpl_plotter.three_d import surface
